Attribute VB_Name = "VBVSL_VisioUtils"
Option Explicit

'-----------------------------------------------------------
' Some useful Visio-specific VB utility routines and consts
'-----------------------------------------------------------

Global Const visEvtAdd = -32768

'---------------------------------------------------
Public Function DocFromArgs(Args As VBVSL_ArgHandler) As Visio.Document
'---------------------------------------------------
Dim DocStr As String, DocIx As Integer

  On Error Resume Next
  Set DocFromArgs = Nothing ' just to be sure
  DocStr = Args.FindValue("doc")
  DocIx = Val(DocStr)
  Set DocFromArgs = glo.VisioApp.Documents(DocIx)
End Function

'---------------------------------------------------
Public Function ShapeFromArgs(Args As VBVSL_ArgHandler) As Visio.Shape
'---------------------------------------------------
Dim ADoc As Visio.Document
Dim PageStr As String, PageIx As Integer, APage As Visio.Page
Dim ShapeNameId As String
  
  On Error Resume Next
  Set ShapeFromArgs = Nothing ' just in case
  Set ADoc = DocFromArgs(Args)
  
  PageStr = Args.FindValue("page")
  PageIx = Val(PageStr)
  Set APage = ADoc.Pages(PageIx)
  
  ShapeNameId = Args.FindValue("shape")
  Set ShapeFromArgs = APage.Shapes(ShapeNameId)
End Function

'---------------------------------------------------
Public Function EnsureUserCellExists(Sheet As Visio.Shape, RowName As String) As Visio.Cell
'---------------------------------------------------
Dim CellName As String, ACell As Visio.Cell
  
  'On Error Resume Next
  CellName = "User." + RowName
  Set EnsureUserCellExists = Nothing ' just in case
  If Sheet.CellExists(CellName, False) Then
    Set EnsureUserCellExists = Sheet.Cells(CellName)
  Else ' add the requisite row
    Sheet.AddNamedRow visSectionUser, RowName, visTagDefault
    Set ACell = Sheet.CellsU(CellName)
    ACell.Formula = ""
    Set EnsureUserCellExists = ACell
  End If
End Function

'---------------------------------------------------
Public Function StringInCell(ACell As Visio.Cell, S As String, AddIt As Boolean) As Boolean
'---------------------------------------------------
Dim CellStr As String, CellStrU, StrPos As Integer, SUC As String
  SUC = UCase$(S)
  StringInCell = False
  If Not (ACell Is Nothing) Then
    CellStr = TrimQuotes(ACell.Formula)
    CellStrU = UCase$(CellStr)
    StrPos = InStr(1, CellStrU, SUC, vbTextCompare)
    If StrPos = 0 Then
      StringInCell = False
      If AddIt Then
        If Len(CellStr) = 0 Then
          CellStr = S
        Else
          CellStr = CellStr + " " + S
        End If
        ACell.FormulaForce = "=""" + CellStr + """"
        StringInCell = True
      End If
    Else
      StringInCell = True
    End If
  End If
End Function

'---------------------------------------------------
Public Function EnsureAOEventExists(AEventList As Visio.EventList, EventCode As Integer, _
                Target As String, TargetArgs As String, Persist As Boolean) As Visio.Event
'---------------------------------------------------
' Check whether an event already exists with same specs, instate if not
Dim AEvent As Visio.Event
Dim TestEventCode As Integer, TestAction As Integer, TestTarget As String
Dim TestTargetArgs As String
  'On Error Resume Next
  Set EnsureAOEventExists = Nothing
  For Each AEvent In AEventList
    TestEventCode = 0
    TestAction = 0
    TestTarget = ""
    TestTargetArgs = ""
    
    ' following kludge needed because Visio exceptions for some combinations
    On Error Resume Next
      TestEventCode = AEvent.Event
      TestAction = AEvent.Action
      TestTarget = AEvent.Target
      TestTargetArgs = AEvent.TargetArgs
    On Error GoTo 0
      
    If TestEventCode = EventCode And _
       TestAction = visActCodeRunAddon And _
       TestTarget = Target And _
       TestTargetArgs = TargetArgs Then GoTo found
  Next AEvent
  Set AEvent = AEventList.Add(EventCode, visActCodeRunAddon, Target, TargetArgs)
found:
  If Persist Then AEvent.Persistent = -1 ' true
  Set EnsureAOEventExists = AEvent
End Function

'----------------------------------------------------
Public Sub DocSubscriptionEnforce(ADoc As Visio.Document, ServiceName As String)
'----------------------------------------------------
' Add Doc rn_SubscribedServices if needed
'   Add string: ServiceName
Dim ASheet As Visio.Shape, ACell As Visio.Cell
  Set ASheet = ADoc.DocumentSheet
  Set ACell = EnsureUserCellExists(ASheet, rn_SubscribedServices)
  StringInCell ACell, ServiceName, True
End Sub

'------------------------------------------------------
Public Sub DocStdPersistentEvents(ADoc As Visio.Document, AddOnName As String)
'------------------------------------------------------
Dim AEventList As Visio.EventList, AEvent As Visio.Event
Dim EventCode As Integer, Cmd As String
  '------------------------------------------------------
  ' Add Doc Persistent events if not present already
  '------------------------------------------------------
  Set AEventList = ADoc.EventList
  
  EventCode = visEvtCodeOpen
  Cmd = "Cmd=" + kw_DocOpened
  Set AEvent = EnsureAOEventExists(AEventList, EventCode, AddOnName, Cmd, True)
  
  EventCode = visEvtCodeCreate
  Cmd = "Cmd=" + kw_DocCreated
  Set AEvent = EnsureAOEventExists(AEventList, EventCode, AddOnName, Cmd, True)
End Sub

'-----------------------------------------------------------
Public Sub ShapeSubscriptionEnforce(AShape As Visio.Shape, ServiceName As String)
'-----------------------------------------------------------
Dim ACell As Visio.Cell
  Set ACell = EnsureUserCellExists(AShape, rn_SubscribedServices)
  StringInCell ACell, ServiceName, True
End Sub

'-----------------------------------------------------------
Public Function IsSubscribed(Sheet As Visio.Shape, ServiceName As String) As Boolean
'-----------------------------------------------------------
Dim ACell As Visio.Cell
  IsSubscribed = False
  On Error Resume Next
    Set ACell = Sheet.CellsU(cn_SubscribedServices)
  On Error GoTo 0
  If ACell Is Nothing Then GoTo skipit
  
  IsSubscribed = StringInCell(ACell, ServiceName, False)
skipit:
End Function
